require 'redis-namespace'
require_relative "nice_cache/fragment"
require_relative "nice_cache/tag"


class NiceCache
  extend Forwardable
  attr_accessor :redis, :tag_store

  COMMANDS_WITH_TAGS = [
    :set,
    :setnx,
    :hmset,
    :hsetnx,
    :lset,
    :sadd,
    :zadd,
    # :mset,
    # :msetnx,
    # :zinterstore,
    # :zunionstore
  ]

  def convert_tags(arg)
    arg.respond_to?(:key) ? arg[:tags] : arg
  end

  COMMANDS_WITH_TAGS.each do |method|
    define_method method, ->(*args) do
      if args.count > Redis.instance_method(method).arity
        tags = convert_tags(args.pop)
        key = args[0]
        Fragment.new(self, key, tags)
      end

      redis.send(method, *args)
    end
  end

  def del(fragment_name)
    Fragment.find(self, fragment_name).delete
  end

  def destroy(key)
    redis.del key
  end

  def initialize(options = {})
    @redis = options[:redis] || Redis.current
    @tag_store = Redis::Namespace.new(:NiceCache, redis: redis)
  end

  def method_missing(method, *args, &block)
    return super unless redis.respond_to?(method)

    self.class.send(:define_method, method, ->(*arguments) {
      redis.send(method, *arguments, &block)
    })

    self.send(method, *args, &block)
  end

  def respond_to_missing?(method_name, include_private = false)
    redis.respond_to?(method_name, include_private) || super
  end

  def sweep_tags(*tag_names)
    Tag.find(self, tag_names).map do |tag|
      tag.sweep
    end
  end

  def flush_index!
    #TODO: Benchmark
    # all_fragments.map { |fragment| fragment.delete }
    tag_store.keys.map { |key| tag_store.del key }
  end

  def all_fragments
    tag_store.smembers Fragment.all
  end

  private

  # def timestamp_key(time)
  #   "Timestamp:#{time}"
  # end
end
