class NiceCache
  class Fragment
    attr_reader :name

    def self.all
      "AllFragments"
    end

    def self.find(nice_cache, name)
      new(nice_cache, name)
    end

    def initialize(nice_cache, name, tag_names = [])
      @nice_cache = nice_cache
      @name = name
      unless tag_names.empty?
        attach_tags(tag_names)
        persist
      end
    end

    def tags
      Tag.find(@nice_cache, tags_list)
    end

    def attach_with(tag)
      store.sadd(tags_for_me, tag.name)
    end

    def delete
      tags.each do |tag|
        tag.remove_from(self)
      end
      # TODO: Timestamps

      destroy
    end

    def ==(other)
      name == other.name
    rescue
      false
    end

    private

    def self.tags_for(name)
      "TagsFor:#{name}"
    end

    def tags_for_me
      self.class.tags_for(@name)
    end

    def persist
      store.sadd(self.class.all, name)
    end

    def attach_tags(tag_names)
      Tag.find(@nice_cache, tag_names).map{ |tag| tag.attach_on(self) }
    end

    def tags_list
      store.smembers(tags_for_me)
    end

    def destroy
      store.srem(self.class.all, name)
      store.del tags_for_me
      @nice_cache.destroy(name)
    end

    def store
      @nice_cache.tag_store
    end

    # def add_timestamp
    #   redis.sadd(NiceCache.all_fragments_key, @name)
    # end
  end
end
