class NiceCache
  class Tag
    attr_reader :name

    def self.find(nice_cache, *tag_names)
      tag_names.flatten.map{ |tag_symbol| new(nice_cache, tag_symbol) }
    end

    def initialize(nice_cache, name)
      @nice_cache = nice_cache
      @name = name.to_s
    end

    def attach_on(fragment)
      save_fragment(fragment.name)
      fragment.attach_with(self)
      self
    end

    def fragments
      fragments_list.map{ |fragment_name| Fragment.find(@nice_cache, fragment_name) }
    end

    def remove_from(fragment)
      store.srem(tag_key, fragment.name)
    end

    def sweep
      fragments.map(&:delete)
      destroy
    end

    def ==(other)
      name == other.name
    rescue
      false
    end

    private

    def store
      @nice_cache.tag_store
    end

    def self.key(name)
      "Tag:#{name}"
    end

    def save_fragment(fragment_name)
      store.sadd(tag_key, fragment_name)
    end

    def tag_key
      self.class.key(@name)
    end

    def destroy
      store.del tag_key
    end

    def fragments_list
      store.smembers(tag_key)
    end
  end
end
